/* @(#)dem68000.dsc	6.16 3/1/93 09:17:59 */

/* Custom disassembler example for 68000 */


%% DEF
long	nopc_addr, dest_address;
int	pipeline;
int	datatr_opcpipe;
int	last_opc;
int	state;
int	new_state;
static char instr_status;

#define	MAXPIPE		3

#define	FC_DATA		0b101
#define	FC_PROGRAM	0b110

#define	SYNC_OK			0
#define	SYNC_LOST		1

%% EQU

/* lookup tables */
LT: STATUS_TAB, decode, ChkNextOpc
LT: Compute8bitBrAddr, FindDataRead, FindDataWrite
LT: CheckBrAddress
LT: ChkAddrSeq
LT: FindAddrSeqBrk
LT: SelUnused
LT: CheckSync
LT: sync_lost

/* index tables */
IT: FindNextOpc, print, ComputeReturnAddr
IT: FindNewOpcAddress

%% FORMAT

logo: "68000 C-DISA Example"

head: "68000 C-DISA Example"  39

tab: 10,32 7

Pods: threshold = {{TTL, TTL},
                   {TTL, TTL},
                   {TTL, TTL},
                   {TTL, TTL}
                  }

clock: udsn = { name = "UDSN",
               edge = rising,
               channel = 31,
               qualifier = { channels = { 52 },
                             required = no,
                             levels = { high }
                           }
             }

clock: ldsn = { name = "LDSN",
               edge = rising,
               channel = 30,
               qualifier = { channels = { 52 },
                             required = no,
                             levels = { high }
                           }
             }

label: DSCTRL =  { name = "DSCTRL",
		 radix = bin,
                 channels = { 29, 28, 27 }
               }

label: FC2_0 =  { name = "FC2_0",
		 radix = bin,
                channels = { 26, 25, 24 }
               }

label: ADDRESS ={ name = "ADDRESS",
                channels = { 23, 22, 21, 20, 19, 18, 17, 16,
			     15, 14, 13, 12, 11, 10,  9,  8,
			      7,  6,  5,  4,  3,  2,  1,  0}
               }

label: DATA =  { name = "DATA",
                channels = { 47, 46, 45, 44, 43, 42, 41, 40,
			     39, 38, 37, 36, 35, 34, 33, 32}
               }


clockseq: SEQ1 = { udsn }
clockseq: SEQ2 = { ldsn }


%% START
! STATUS_TAB !


%% STATUS_TAB
(instr_status = SYNC_LOST)	    ! sync_lost { instr_status = SYNC_OK } !
(FC2_0 = FC_PROGRAM, DSCTRL=0b100)  ! decode !
(FC2_0 = FC_DATA,    DSCTRL=0b1..)  ! MR !	/* memory read */
(FC2_0 = FC_DATA,    DSCTRL=0b0..)  ! MW !	/* memory write */
(FC2_0 = 0b...)			    ! "reserved function code" !

%% decode	/* _ in a condition is only used for readability */
(DATA = 0b0011_0110_0011_1100)	 ! "MOVE.W\t#" FindNextOpc[0] PROG \
				   print[0] ",D3" !
(DATA = 0b0011_0010_0001_0010)	 ! "MOVE.W\t(A2),D1" FindDataRead MR !
(DATA = 0b0011_1010_1000_0001)	 ! "MOVE.W\tD1,(A5)" FindDataWrite MW !
(DATA = 0b0010_0110_0111_1100)	 ! "MOVEA.L\t#" FindNextOpc[0] PROG \
				   print[0] FindNextOpc[0] PROG \
				   print[0] ",A3"!
(DATA = 0b0110_0001_...._....)	 ! "BSR\t" \
				   Compute8bitBrAddr print[1]  \
				   { last_opc = TELL } \
				   FindNewOpcAddress[0] CheckBrAddress \
				   CheckSync \
				   GOTO[last_opc] \
				   FindDataWrite MW NEXT FindDataWrite MW !
(DATA = 0b0101_[...]1_0100_0011) ! "SUBQ.W\t#%d,D3" !
(DATA = 0b0100_1110_0111_0101)	 ! "RTS\t" 	\
				   { last_opc = TELL } \
				   FindDataRead \
				   ComputeReturnAddr[0] \
				   print[1] \
				   GOTO[last_opc] \
				   FindNewOpcAddress[0] CheckBrAddress \
				   CheckSync !
(DATA = 0b0110_0111_...._....)	 ! "BEQ\t" \
				   Compute8bitBrAddr print[1] \
				   FindNewOpcAddress[0] CheckBrAddress !
(DATA = 0b0110_0110_...._....)	 ! "BNE\t" \
				   Compute8bitBrAddr print[1] \
				   FindNewOpcAddress[0] CheckBrAddress !

%% FindNextOpc
(ADDRESS=0x[......])	         ! { nopc_addr = $1 + 2 } NEXT ChkNextOpc !


%% ChkNextOpc	/* lookup table */
(ADDRESS = nopc_addr, FC2_0 = FC_PROGRAM, DSCTRL = 0b100) ! /* found next opc */ !
(FC2_0 != FC_PROGRAM)	         ! NEXT ChkNextOpc !
	    /* if we didn't find the next opcode function-code 6 is illegal
	       The disassembler looses his synchronisation status */


%% Compute8bitBrAddr	/* data label contains 8-bit signed offset to ADDRESS */
(DATA=0b<.>8_0[<.>7], ADDRESS=0x[......])   ! /* positive offset */ \
		{dest_address = $2 + $1;     /* compute destination address */ \
		 dest_address = dest_address + 2 } /* adjust ADDRESS value */ \
		"+%d"                         /* print offset */ !
(DATA=0b<.>8_1[<.>7], ADDRESS=0x[......])   ! /* negative offset */ \
		{$1 = $1 - 0x80; 	      /* sign extend $1 */ \
		 dest_address = $2 + $1;      /* compute destination address */ \
		 dest_address = dest_address + 2 } /* adjust ADDRESS value */ \
		"%d"                          /* print offset */ !


%% print
(DATA=0x[....])	                 ! "%04x"     !
(dest_address = 0x..[......])	 ! "\t{%06x}" !


%% FindDataRead
(FC2_0 = FC_DATA, DSCTRL = 0b1.., datatr_opcpipe > 1)	 ! /* found */ !
(FC2_0 = FC_PROGRAM)	! /* not found yet */ \
			  { datatr_opcpipe = datatr_opcpipe + 1 } \
			  NEXT FindDataRead !
(FC2_0 = 0b...)		! /* not found yet */ NEXT FindDataRead !

 
%% FindDataWrite
(FC2_0 = FC_DATA, DSCTRL = 0b0.., datatr_opcpipe > 1)	 ! /* found */ !
(FC2_0 = FC_PROGRAM)	! /* not found yet */ \
			  { datatr_opcpipe = datatr_opcpipe + 1 } \
			  NEXT FindDataWrite !
(FC2_0 = 0b...)		! /* not found yet */ NEXT FindDataWrite !


%% ComputeReturnAddr
(DATA=0x[....])		! { dest_address = $1 << 16 } MR NEXT FindDataRead \
			  ComputeReturnAddr[1] !
(DATA=0x[....])		! { dest_address = dest_address + $1 } MR !


%% FindNewOpcAddress
! FindNextOpc[0] { state = TELL } FindAddrSeqBrk !


%% FindAddrSeqBrk
(pipeline > MAXPIPE)	! /* not found, reposition */ GOTO[state] !
(ADDRESS=0b[<.>24])     ! { nopc_addr = $1; pipeline = pipeline + 1 } \
			  NEXT ChkAddrSeq !


%% ChkAddrSeq
(ADDRESS = nopc_addr, FC2_0 = FC_PROGRAM, DSCTRL = 0b1..) ! FindAddrSeqBrk !
(FC2_0 != FC_PROGRAM)	! NEXT ChkAddrSeq !
(FC2_0 = FC_PROGRAM)	! /* found addr sequence break, ready */ !


%% CheckBrAddress
(ADDRESS = dest_address /* dest_address computed before */ )\
			/* branch executed */ \
			! { new_state = TELL} GOTO[state] SelUnused !
(ADDRESS = 0x......)	! /* branch not executed */ !


%% SelUnused
(state < new_state, FC2_0 = FC_PROGRAM) ! UNUSED NEXT { state = TELL } SelUnused !
(state = new_state)	! /* ready */ !
(state = 0x....)	! NEXT { state = TELL } SelUnused !

%% CheckSync
(ADDRESS = dest_address)	! { instr_status = SYNC_OK } !
(ADDRESS != dest_address)	! { instr_status = SYNC_LOST } !

%% sync_lost
(instr_status = SYNC_OK)	! { instr_status = SYNC_LOST } sync_lost !
