/*-----------------------------------------------------------*/
/*                                                           */
/*  Cdisa for disassembling the i2c-bus signals              */
/*                                                           */ 
/*  author: Rini van Dijk                                    */
/*  dep.  : development DTS - IE                             */
/*          Eindhoven                        date: 23-07-92  */
/*                                                           */
/*-----------------------------------------------------------*/
%%DEF

int  tmp;

%%EQU

LT: START_STOP,ADDR_DATA, STRT, ADDR, DATA, ACK, L_10BADR

%%FORMAT

logo: " I2C "
head: " S/P    ADDR  R/WN  DATA  ACK " 30

/* --------------------------------------
        
        start 
               3aa   
                     wr    7e    ack
        stop
        start     
                2b
                     rd    6f    ack
                     rd    23    ack
                     rd    17   nack
        stop
        start
                50
                     wr    00    ack
                     wr    01    ack
                     wr    03    ack
                     wr    40    ack
        stop
        start
                general call address
        stop
--------------------------------------- */

pods:threshold={{TTL,TTL}
               }

clock:ST_CLK  ={ name     ="ST_CLK",
                 edge     = any,
                 channel  = 13,
                 display  = state,
                 timing   = none,
                 polarity = +
               }

clock:D_CLK   ={ name     ="D_CLK",
                 edge     = falling,
                 channel  = 12,
                 display  = state,
                 timing   = none,
                 polarity = +
               }

label:DATA    ={ name     ="DATA",
                 channels = { 7, 6, 5, 4, 3, 2, 1, 0 },
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               }

label:ACK     ={ name     ="ACK",
                 channels = {8},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               }

label:RWN     ={ name     ="RWN",
                 channels = {9},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               } 

label:DTADRN  ={ name     ="DTADRN",
                 channels = {10},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {D_CLK},
                 required = yes
               } 

label:STSP    ={ name     ="STSP",
                 channels = {11},
                 display  = state,
                 timing   = none,
                 polarity = +,
                 radix    = bin,
                 clocks   = {ST_CLK},
                 required = yes
               } 

label:SCL     ={ name     ="SCL",
                 channels = {14},
                 timing   = dataandtrigger,
                 display  = timing,
                 polarity = +,
                 required = no
               }

label:SDA     ={ name     ="SDA",
                 channels = {15},
                 timing   = dataandtrigger,
                 display  = timing,
                 polarity = +,
                 required = no
               }

clockseq:STRTSTP = {ST_CLK}
clockseq:DTA     = {D_CLK}

%%START

!STRT!

/*-----------------------------------------------------------*/
/*   defining start en stop conditions                       */
/*-----------------------------------------------------------*/
%%STRT
(STRTSTP)                ! START_STOP  !
(DTA)                    ! ADDR_DATA   !

%%START_STOP
( STSP = 0b1 )           ! " start  "   !
( STSP = 0b0 )           ! " stop   "   !

/*-----------------------------------------------------------*/
/*   test if data_byte is address or data                    */
/*-----------------------------------------------------------*/
%%ADDR_DATA
( DTADRN = 0b0 )         !    ADDR      !
( DTADRN = 0b1 )         !    DATA      !

/*-----------------------------------------------------------*/
/*   if address then test if read or write and display       */
/*-----------------------------------------------------------*/
%%ADDR

( DATA = 0b[0000000]_0 )   ! "        general call address " !
( DATA = 0b[0000000]_1 )   ! "        START  byte          " !
( DATA = 0b[0000001]_. )   ! "        CBUS   address       " !
( DATA = 0b[0000010]_. )   ! "        reserved bus format  " !
( DATA = 0b[0000011]_. )   ! "        for future purposes  " !
( DATA = 0b[00001..]_. )   ! "        for future purposes  " !
( DATA = 0b11110[..]0 )    ! {tmp=$1; tmp=tmp<< 8} next L_10BADR ! 
( DATA = 0b[11111..]_. )   ! "        for future purposes  " !

( DATA = 0b[<.>7]_.)       ! " \t   %02x " !

/*-----------------------------------------------------------*/
/*   display address if address is 10bit address             */
/*-----------------------------------------------------------*/
%%L_10BADR
( DATA = 0x[..] )          !  { $1=$1+tmp } " \t  %03x " !

/*-----------------------------------------------------------*/
/*   if data then display data, direction  and acknowledge   */
/*-----------------------------------------------------------*/
%%DATA
( DATA = 0x[..], RWN = 0b0 )   ! " wr   \t\t      %02x"  ACK !
( DATA = 0x[..], RWN = 0b1 )   ! " rd   \t\t      %02x"  ACK !

%%ACK
( ACK  = 0b0 )           ! "    ack"  !
( ACK  = 0b1 )           ! "    nack" !
